@@ -35,7 +35,9 @@ class SqlDiff(aegis.database.Row):
 
     @staticmethod
     def insert(sql_diff_name):
-        sql = 'INSERT INTO sql_diff (sql_diff_name) VALUES (%s) RETURNING sql_diff_id'
+        sql = 'INSERT INTO sql_diff (sql_diff_name) VALUES (%s)'
+        if type(db()) is aegis.database.PostgresConnection:
+            sql += ' RETURNING sql_diff_id'
         return db().execute(sql, sql_diff_name)
 
     @classmethod
@@ -99,19 +101,27 @@ class UserAgent(aegis.database.Row):
         sql = 'UPDATE user_agent SET robot_ind=%s WHERE user_agent_id=%s'
         return db().execute(sql, robot_ind, user_agent_id)
 
+    def set_ua_json(self, ua_json):
+        if self['user_agent_json'] or not ua_json:
+            return
+        sql = "UPDATE user_agent SET user_agent_json=%s WHERE user_agent_id=%s"
+        return db().execute(sql, ua_json, self['user_agent_id'])
+
 
 class User(aegis.database.Row):
-    table_name = 'user_' if aegis.config.get('pg_database') else 'user'
+    table_name = 'user_' if aegis.database.pgsql_available else 'user'
     table_names = {'pgsql': 'user_', 'mysql': 'user'}
     id_column = 'user_id'
 
-    @staticmethod
-    def insert(user_agent_id):
-        sql = 'INSERT INTO user_ (user_agent_id) VALUES (%s) RETURNING user_id'
+    @classmethod
+    def insert(cls, user_agent_id):
+        sql = 'INSERT INTO '+cls.table_name+' (user_agent_id) VALUES (%s)'
+        if type(db()) is aegis.database.PostgresConnection:
+            sql += ' RETURNING user_id'
         return db().execute(sql, user_agent_id)
 
     def set_member_id(self, member_id):
-        sql = 'UPDATE user_ SET member_id=%s WHERE user_id=%s'
+        sql = 'UPDATE '+self.table_name+' SET member_id=%s WHERE user_id=%s'
         return db().execute(sql, member_id, self['user_id'])
 
 
@@ -121,7 +131,9 @@ class Email(aegis.database.Row):
 
     @staticmethod
     def insert(email):
-        sql = "INSERT INTO email (email) VALUES (%s) RETURNING email_id"
+        sql = "INSERT INTO email (email) VALUES (%s)"
+        if type(db()) is aegis.database.PostgresConnection:
+            sql += ' RETURNING email_id'
         return db().execute(sql, email)
 
     @classmethod
@@ -153,7 +165,9 @@ class Member(aegis.database.Row):
 
     @staticmethod
     def insert(email_id):
-        sql = "INSERT INTO member (email_id) VALUES (%s) RETURNING member_id"
+        sql = "INSERT INTO member (email_id) VALUES (%s)"
+        if type(db()) is aegis.database.PostgresConnection:
+            sql += ' RETURNING member_id'
         return db().execute(sql, email_id)
 
     @classmethod
@@ -261,7 +275,9 @@ class EmailTracking(aegis.database.Row):
 
     @staticmethod
     def insert(email_type_id, from_email_id, to_email_id, email_data):
-        sql = "INSERT INTO email_tracking (email_type_id, from_email_id, to_email_id, email_uuid, email_data, send_dttm) VALUES (%s, %s, %s, %s, %s, NOW()) RETURNING email_tracking_id"
+        sql = "INSERT INTO email_tracking (email_type_id, from_email_id, to_email_id, email_uuid, email_data, send_dttm) VALUES (%s, %s, %s, %s, %s, NOW())"
+        if type(db()) is aegis.database.PostgresConnection:
+            sql += ' RETURNING email_tracking_id'
         return db().execute(sql, email_type_id, from_email_id, to_email_id, uuid.uuid4().hex, email_data)
 
     def mark_sent(self):
@@ -275,7 +291,9 @@ class EmailLink(aegis.database.Row):
 
     @staticmethod
     def insert(email_id):
-        sql = "INSERT INTO email_link (email_id, magic_token) VALUES (%s, %s) RETURNING email_link_id"
+        sql = "INSERT INTO email_link (email_id, magic_token) VALUES (%s, %s)"
+        if type(db()) is aegis.database.PostgresConnection:
+            sql += ' RETURNING email_link_id'
         return db().execute(sql, email_id, aegis.stdlib.magic_token())
 
     def mark_accessed(self):
@@ -308,8 +326,10 @@ class Pageview(aegis.database.Row):
 
     @classmethod
     def insert(cls, user_id, member_id, url_path, url_query, url_tx, request_name):
-      sql = "INSERT INTO pageview (user_id, member_id, url_path, url_query, url_tx, request_name) VALUES (%s, %s, %s, %s, %s, %s) RETURNING pageview_id"
-      return db().execute(sql, user_id, member_id, url_path, url_query, url_tx, request_name)
+        sql = "INSERT INTO pageview (user_id, member_id, url_path, url_query, url_tx, request_name) VALUES (%s, %s, %s, %s, %s, %s)"
+        if type(db()) is aegis.database.PostgresConnection:
+            sql += ' RETURNING pageview_id'
+        return db().execute(sql, user_id, member_id, url_path, url_query, url_tx, request_name)
 
 
 class HydraType(aegis.database.Row):
@@ -456,13 +476,13 @@ class HydraQueue(aegis.database.Row):
         sql = 'UPDATE hydra_queue SET try_cnt=try_cnt+1 WHERE hydra_queue_id=%s'
         return db().execute(sql, self['hydra_queue_id'])
 
-    def incr_error_cnt(self):
+    def incr_error_cnt(self, minutes=1):
         self['error_cnt'] += 1
         if type(db()) is aegis.database.PostgresConnection:
-            sql = "UPDATE hydra_queue SET error_cnt=error_cnt+1, start_dttm=NULL, work_dttm=work_dttm + INTERVAL '1 MINUTE' WHERE hydra_queue_id=%s"
+            sql = "UPDATE hydra_queue SET error_cnt=error_cnt+1, start_dttm=NULL, work_dttm=NOW() + INTERVAL '%s MINUTE' WHERE hydra_queue_id=%s"
         elif type(db()) is aegis.database.MysqlConnection:
-            sql = "UPDATE hydra_queue SET error_cnt=error_cnt+1, start_dttm=NULL, work_dttm=work_dttm + INTERVAL 1 MINUTE WHERE hydra_queue_id=%s"
-        return db().execute(sql, self['hydra_queue_id'])
+            sql = "UPDATE hydra_queue SET error_cnt=error_cnt+1, start_dttm=NULL, work_dttm=NOW() + INTERVAL %s MINUTE WHERE hydra_queue_id=%s"
+        return db().execute(sql, minutes, self['hydra_queue_id'])
 
     def start(self):
         hydra_type = HydraType.get_id(self['hydra_type_id'])
@@ -514,7 +534,9 @@ class ReportType(aegis.database.Row):
 
     @classmethod
     def insert(cls, report_type_name, report_sql):
-        sql = "INSERT INTO report_type (report_type_name, report_sql) VALUES (%s, %s) RETURNING report_type_id"
+        sql = "INSERT INTO report_type (report_type_name, report_sql) VALUES (%s, %s)"
+        if type(db()) is aegis.database.PostgresConnection:
+            sql += ' RETURNING report_type_id'
         return db().execute(sql, report_type_name, report_sql)
 
     @classmethod
@@ -580,6 +602,10 @@ class Build(aegis.database.Row):
         sql = "UPDATE build SET build_size=%s WHERE build_id=%s AND build_size IS NULL"
         return db().execute(sql, build_size, self['build_id'])
 
+    def set_message(self, message, build_step):
+        sql = "UPDATE build SET " + build_step + "_message=%s WHERE build_id=%s AND " + build_step + "_message IS NULL"
+        return db().execute(sql, message, self['build_id'])
+
     def set_deployed(self):
         sql = "UPDATE build SET deploy_dttm=NOW() WHERE build_id=%s AND deploy_dttm IS NULL"
         return db().execute(sql, self['build_id'])
@@ -612,6 +638,46 @@ class Build(aegis.database.Row):
         return db().query(sql, cls=cls)
 
     @classmethod
-    def scan_stale_builds(cls):
-        sql = "SELECT * FROM build WHERE deploy_dttm IS NOT NULL ORDER BY deploy_dttm DESC"
+    def scan_stale_builds(cls, env):
+        sql = "SELECT * FROM build WHERE env=%s AND deploy_dttm IS NOT NULL ORDER BY deploy_dttm DESC"
+        return db().query(sql, env, cls=cls)
+
+    @classmethod
+    def deployed_envs(cls):
+        sql = "SELECT DISTINCT env FROM build WHERE delete_dttm IS NULL"
         return db().query(sql, cls=cls)
+
+
+class Cache(aegis.database.Row):
+    table_name = 'cache'
+    id_column = 'cache_id'
+
+    @staticmethod
+    def insert(cache_key, cache_json, cache_expiry):
+        sql = "INSERT INTO cache (cache_key, cache_json, cache_expiry) VALUES (%s, %s, %s)"
+        if type(db()) is aegis.database.PostgresConnection:
+            sql += ' RETURNING cache_id'
+        try:
+            return db().execute(sql, cache_key, cache_json, cache_expiry)
+        except aegis.database.PgsqlUniqueViolation as ex:
+            logging.warning("Ignoring duplicate key in cache")
+
+    @classmethod
+    def get_key(cls, cache_key):
+        sql = "SELECT * FROM cache WHERE cache_key=%s"
+        return db().get(sql, cache_key, cls=cls)
+
+    @staticmethod
+    def update_key(cache_key, cache_json, cache_expiry):
+        sql = "UPDATE cache SET cache_json=%s, cache_expiry=%s WHERE cache_key=%s"
+        return db().execute(sql, cache_json, cache_expiry, cache_key)
+
+    @staticmethod
+    def del_key(cache_key):
+        sql = "DELETE FROM cache WHERE cache_key=%s"
+        return db().execute(sql, cache_key)
+
+    @staticmethod
+    def purge_expired():
+        sql = "DELETE FROM cache WHERE cache_expiry < NOW()"
+        return db().execute(sql)
